{%- macro generateRequestValidation() -%}
{%- set operations = (api | resolveOperations) -%}
{%- set typeDependencies = ["ResponseData"] -%}
{%- set signatureDependencies = [] -%}
{%- set validationDependencies = [] -%}

{%- for operation in operations -%}
    {%- if operation.parameters -%}
        {%- set signatureDependencies = (signatureDependencies.push(operation.name + "ParametersType"), signatureDependencies) -%}
        {%- for parameter in operation.parameters -%}
            {%- set validationDependencies = (validationDependencies.push("validate" + (parameter.name | formatName)), validationDependencies) -%}
        {%- endfor -%}
    {%- endif -%}
    {%- if operation.requestBody -%}
        {%- set signatureDependencies = (signatureDependencies.push(operation.name + "RequestBodyType"), signatureDependencies) -%}
        {%- set validationDependencies = (validationDependencies.push("validate" + (operation.name | cap) + "RequestBody"), validationDependencies) -%}
    {%- endif -%}
    {%- if operation.responses -%}
        {%- set signatureDependencies = (signatureDependencies.push(operation.name + "ResponseType"), signatureDependencies) -%}
        {%- for response in operation.responses -%}
            {%- if response.schema -%}
                {%- set validationDependencies = (validationDependencies.push("validate" + (operation.name | cap) + "Response" + response.status), validationDependencies) -%}
            {%- endif -%}
            {%- for header in response.headers -%}
                {%- set validationDependencies = (validationDependencies.push("validate" + (operation.name | cap) + "Header" + (header.name)), validationDependencies) -%}
            {%- endfor -%}
        {%- endfor -%}
    {%- endif -%}
{%- endfor -%}

/**
 * This file was automatically generated by openapi-codegeneration.
 * DO NOT MODIFY IT BY HAND. Instead, modify the source OpenAPI file,
 * and run openapi-codegeneration to regenerate this file.
 */
import {
    {{ typeDependencies | unique | join(",\n\t") }}
} from "./types"

import {
    {{ signatureDependencies | unique | join(",\n\t") }}
} from "./signatures"

import {
    {{ validationDependencies | unique | join(",\n\t") }}
} from "./basicValidation"

{%- for operation in operations %}

/**
 * This function validates the inputs of a {{ operation.method | upper }} request on {{ operation.path }}
 */
export function validate{{ operation.name | cap }}Input(
    {{- "parameters: " + operation.name + "ParametersType" if operation.parameters -}}
    {{- ", " if (operation.parameters and operation.requestBody) -}}
    {{- "body: " + operation.name + "RequestBodyType" if operation.requestBody -}}
) {
    {% if operation.parameters -%}
    if (parameters) {
        {%- for parameter in operation.parameters %}
        if (
            {{ "parameters['" + parameter.name + "'] && " if not parameter.required -}}
            !validate{{ parameter.name | formatName -}}(parameters["{{ parameter.name }}"])
        ) {
            (validate{{ operation.name | cap }}Input as any).errors = (validate{{ parameter.name | formatName }} as any).errors
            return false
        }
        {%- endfor %}
    }

    {% endif -%}

    {% if operation.requestBody -%}
    if ({{ "body && " if not operation.requestBody.required -}}!validate{{ operation.name | cap }}RequestBody(body)) {
        (validate{{ operation.name | cap }}Input as any).errors = (validate{{ operation.name | cap }}RequestBody as any).errors
        return false
    }

    {% endif -%}

    return true
}

/**
 * This function validates the outputs to a {{ operation.method | upper }} request on {{ operation.path }}
 */
export function validate{{ operation.name | cap }}Output(response: ResponseData): response is {{ operation.name }}ResponseType {
    if (response.status < 100 || response.status >= 600) return false

    {% for response in operation.responses -%}
    {%- if response.status | endswith("XX") %}
    if (response.status >= {{ (response.status | replace("X","0") | int) }} && response.status < {{ (response.status | replace("X","0") | int) + 100 }} ) {
    {%- else %}
    if (response.status === {{ response.status }}) {
    {%- endif %}
        {% if response.headers -%}
        if (response.headers) {
            {%- for header in response.headers %}
            if (
                {{ "response.headers['" + header.name + "'] && " if not header.required -}} 
                !validate{{ operation.name | cap }}Header{{ header.name }}(response.headers["{{ header.name }}"])
            ) {
                (validate{{ operation.name | cap }}Output as any).errors = (validate{{ operation.name | cap }}Header{{ header.name }} as any).errors
                return false
            }   
            {%- endfor %}
        }

        {% endif -%}
        {% if response.schema -%}
        if (!validate{{ operation.name | cap }}Response{{ response.status }}(response.body)) {
            (validate{{ operation.name | cap }}Output as any).errors = (validate{{ operation.name | cap }}Response{{ response.status }} as any).errors
            return false
        }

        {% endif -%}    
        return true
    }

    {% endfor -%}
    (validate{{ operation.name | cap }}Output as any).errors = `Response status ${response.status} is unexpected`

    return false
}

{%- endfor %}
{%- endmacro -%}
{{ generateRequestValidation() | prettier }}